"""insights — reproducible metrics + matplotlib visualizations for sklearn estimators.

Quickstart
----------
>>> from insights import generate_report
>>> outputs = generate_report(clf, X_test, y_test, output_dir="reports/", prefix="digits")
>>> print(outputs)  # dict mapping role → absolute Path
"""
from ._report import generate_report
from ._metrics import (
    compute_classification_metrics,
    compute_regression_metrics,
    compute_georf_metrics,
)
from ._plots import (
    plot_confusion_matrix,
    plot_roc_curves,
    plot_feature_importances,
    plot_actual_vs_predicted,
    plot_residuals,
    plot_residual_distribution,
    plot_tree_quality_scores,
)

__all__ = [
    "generate_report",
    "compute_classification_metrics",
    "compute_regression_metrics",
    "compute_georf_metrics",
    "plot_confusion_matrix",
    "plot_roc_curves",
    "plot_feature_importances",
    "plot_actual_vs_predicted",
    "plot_residuals",
    "plot_residual_distribution",
    "plot_tree_quality_scores",
]
